Package: rjson
Version: 0.2.23
Title: JSON for R
Author: Alex Couture-Beil [aut, cre]
Authors@R: person(given = "Alex", family = "Couture-Beil", role = c("aut", "cre"), email = "rjson_pkg@mofo.ca")
Maintainer: Alex Couture-Beil <rjson_pkg@mofo.ca>
Depends: R (>= 4.0.0)
Description: Converts R object into JSON objects and vice-versa.
URL: https://github.com/alexcb/rjson
License: GPL-2
Repository: CRAN
Packaged: 2024-09-16 17:34:28 UTC; root
NeedsCompilation: yes
Date/Publication: 2024-09-16 18:20:02 UTC
